@@ -57,6 +57,7 @@ from testlib import VdsmTestCase as TestCaseBase
 from testlib import permutations, expandPermutations
 from testlib import find_xml_element
 from testlib import make_config
+from testlib import recorded
 from testlib import XMLTestCase
 from vdsm import host
 from vdsm import utils
@@ -758,12 +759,12 @@ class TestVm(XMLTestCase):
                 <ioTune>
                     <device name='test-device-by-name'>
                         <maximum>
-                            <totalBytes>9999</totalBytes>
+                            <total_bytes_sec>9999</total_bytes_sec>
                         </maximum>
                     </device>
                     <device name='other-device'>
                         <maximum>
-                            <totalBytes>9999</totalBytes>
+                            <total_bytes_sec>9999</total_bytes_sec>
                         </maximum>
                     </device>
                 </ioTune>
@@ -808,12 +809,8 @@ class TestVm(XMLTestCase):
 
             expected_xml = (u"""
             <qos>
+                <vcpuLimit>50</vcpuLimit>
                 <ioTune>
-                    <device name="other-device">
-                        <maximum>
-                            <totalBytes>9999</totalBytes>
-                        </maximum>
-                    </device>
                     <device name="test-device-by-name">
                         <maximum>
                             <total_bytes_sec>200</total_bytes_sec>
@@ -832,6 +829,11 @@ class TestVm(XMLTestCase):
                             <write_iops_sec>105</write_iops_sec>
                         </guaranteed>
                     </device>
+                    <device name="other-device">
+                        <maximum>
+                            <total_bytes_sec>9999</total_bytes_sec>
+                        </maximum>
+                    </device>
                     <device path="test-device-by-path">
                         <maximum>
                             <total_bytes_sec>400</total_bytes_sec>
@@ -851,7 +853,6 @@ class TestVm(XMLTestCase):
                         </guaranteed>
                     </device>
                 </ioTune>
-                <vcpuLimit>50</vcpuLimit>
             </qos>
             """)
 
@@ -866,28 +867,29 @@ class TestVm(XMLTestCase):
                 <ioTune>
                     <device name='test-device-by-name'>
                         <maximum>
-                            <totalBytes>9999</totalBytes>
+                            <total_bytes_sec>9999</total_bytes_sec>
                         </maximum>
                     </device>
                     <device name='other-device'>
                         <guaranteed>
-                            <totalBytes>9999</totalBytes>
+                            <total_bytes_sec>9999</total_bytes_sec>
                         </guaranteed>
                     </device>
                 </ioTune>
             </qos>
             """
             machine._dom = dom
+            machine._updateIoTuneInfo()
 
             tunables = machine.getIoTunePolicyResponse()
             expected = [
                 {'name': u'test-device-by-name',
                  'maximum': {
-                     u'totalBytes': 9999
+                     u'total_bytes_sec': 9999
                  }},
                 {'name': u'other-device',
                  'guaranteed': {
-                     u'totalBytes': 9999
+                     u'total_bytes_sec': 9999
                  }}
             ]
             self.assertEqual(tunables['ioTunePolicyList'], expected)
@@ -1777,6 +1779,165 @@ class FreezingUnexpectedErrorTests(TestCaseBase):
                                              message="fake error"))
 
 
+class BlockIoTuneTests(TestCaseBase):
+
+    def setUp(self):
+        self.iotune_low = {
+            'total_bytes_sec': 0,
+            'read_bytes_sec': 1000,
+            'write_bytes_sec': 1000,
+            'total_iops_sec': 0,
+            'write_iops_sec': 0,
+            'read_iops_sec': 0
+        }
+        self.iotune_high = {
+            'total_bytes_sec': 0,
+            'read_bytes_sec': 2000,
+            'write_bytes_sec': 2000,
+            'total_iops_sec': 0,
+            'write_iops_sec': 0,
+            'read_iops_sec': 0
+        }
+        self.drive = FakeBlockIoTuneDrive('vda', path='/fake/path/vda')
+
+        self.dom = FakeBlockIoTuneDomain()
+        self.dom.iotunes = {self.drive.name: self.iotune_low.copy()}
+
+    @MonkeyPatch(vm, 'isVdsmImage', lambda *args: True)
+    @MonkeyPatch(utils, 'isBlockDevice', lambda *args: False)
+    def test_get_fills_cache(self):
+        with fake.VM() as testvm:
+            testvm._dom = self.dom
+            testvm._devices[hwclass.DISK] = (self.drive,)
+
+            res = testvm.getIoTuneResponse()
+            self.assertFalse(response.is_error(res))
+            self.assertEqual(
+                self.dom.__calls__,
+                [('blockIoTune',
+                    (self.drive.name, libvirt.VIR_DOMAIN_AFFECT_LIVE),
+                    {})]
+            )
+
+            res = testvm.getIoTuneResponse()
+            self.assertFalse(response.is_error(res))
+            self.assertEqual(
+                self.dom.__calls__,
+                [('blockIoTune',
+                    (self.drive.name, libvirt.VIR_DOMAIN_AFFECT_LIVE),
+                    {})]
+            )
+
+    @MonkeyPatch(vm, 'isVdsmImage', lambda *args: True)
+    @MonkeyPatch(utils, 'isBlockDevice', lambda *args: False)
+    def test_set_updates_cache(self):
+        with fake.VM() as testvm:
+            testvm._dom = self.dom
+            testvm._devices[hwclass.DISK] = (self.drive,)
+
+            tunables = [
+                {"name": self.drive.name, "ioTune": self.iotune_high}
+            ]
+
+            res = testvm.getIoTuneResponse()
+            self.assert_iotune_in_response(res, self.iotune_low)
+
+            testvm.setIoTune(tunables)
+
+            res = testvm.getIoTuneResponse()
+            self.assert_iotune_in_response(res, self.iotune_high)
+
+            self.assertEqual(len(self.dom.__calls__), 2)
+            self.assert_nth_call_to_dom_is(0, 'blockIoTune')
+            self.assert_nth_call_to_dom_is(1, 'setBlockIoTune')
+
+    @MonkeyPatch(vm, 'isVdsmImage', lambda *args: True)
+    @MonkeyPatch(utils, 'isBlockDevice', lambda *args: False)
+    def test_set_fills_cache(self):
+        with fake.VM() as testvm:
+            testvm._dom = self.dom
+            testvm._devices[hwclass.DISK] = (self.drive,)
+
+            tunables = [
+                {"name": self.drive.name, "ioTune": self.iotune_high}
+            ]
+
+            testvm.setIoTune(tunables)
+
+            res = testvm.getIoTuneResponse()
+            self.assert_iotune_in_response(res, self.iotune_high)
+
+            self.assertEqual(len(self.dom.__calls__), 1)
+            self.assert_nth_call_to_dom_is(0, 'setBlockIoTune')
+
+    @MonkeyPatch(vm, 'isVdsmImage', lambda *args: True)
+    @MonkeyPatch(utils, 'isBlockDevice', lambda *args: False)
+    def test_cold_cache_set_preempts_get(self):
+        with fake.VM() as testvm:
+            testvm._dom = self.dom
+            testvm._devices[hwclass.DISK] = (self.drive,)
+
+            def _interleaved_update():
+                # this will run in the middle of getIoTuneResponse()
+                tunables = [
+                    {"name": self.drive.name, "ioTune": self.iotune_high}
+                ]
+                testvm.setIoTune(tunables)
+
+            self.dom.callback = _interleaved_update
+            self.assert_iotune_in_response(
+                testvm.getIoTuneResponse(),
+                self.iotune_low
+            )
+
+            self.assertEqual(
+                self.dom.iotunes,
+                {self.drive.name: self.iotune_high}
+            )
+
+    def assert_nth_call_to_dom_is(self, nth, call):
+        self.assertEqual(self.dom.__calls__[nth][0], call)
+
+    def assert_iotune_in_response(self, res, iotune):
+        self.assertEqual(
+            res['ioTuneList'][0]['ioTune'], iotune
+        )
+
+
+class FakeBlockIoTuneDrive(object):
+
+    def __init__(self, name, path=None):
+        self.name = name
+        self.path = path or os.path.join('fake', 'path', name)
+        self.iotune = {}
+        self.specParams = {}
+        self._deviceXML = ''
+
+    def _validateIoTuneParams(self, ioTuneParams):
+        pass
+
+    def getXML(self):
+        return vmxml.Element('fake')
+
+
+class FakeBlockIoTuneDomain(object):
+
+    def __init__(self):
+        self.iotunes = {}
+        self.callback = None
+
+    @recorded
+    def blockIoTune(self, name, flags=0):
+        ret = self.iotunes.get(name, {}).copy()
+        if self.callback is not None:
+            self.callback()
+        return ret
+
+    @recorded
+    def setBlockIoTune(self, name, iotune, flags=0):
+        self.iotunes[name] = iotune.copy()
+
+
 @expandPermutations
 class SyncGuestTimeTests(TestCaseBase):
 
