@@ -1,5 +1,5 @@
 #
-# Copyright 2009-2016 Red Hat, Inc.
+# Copyright 2009-2017 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ import image
 from vdsm import qemuimg
 from vdsm import utils
 
+from vdsm.common.threadlocal import vars
 from vdsm.storage import clusterlock
 from vdsm.storage import constants as sc
 from vdsm.storage import exception as se
@@ -34,7 +35,6 @@ from vdsm.storage import fileUtils
 from vdsm.storage import guarded
 from vdsm.storage import misc
 from vdsm.storage.misc import deprecated
-from vdsm.storage.threadlocal import vars
 from vdsm.storage.volumemetadata import VolumeMetadata
 
 import sd
